@@ -29,12 +29,14 @@
 #  include <config.h>
 #endif
 
+#define SILENT_WINDOW_OPEN_LIMIT 5
+
+#include <signal.h>
+
 #include "gsearchtool.h"
 #include "gsearchtool-support.h"
 #include "gsearchtool-callbacks.h"
 
-#include <signal.h>
-
 static GnomeUIInfo popup_menu[] = {
 	GNOMEUIINFO_ITEM_STOCK (N_("_Open"), 
 				NULL, 
@@ -52,6 +54,8 @@ static GnomeUIInfo popup_menu[] = {
 	GNOMEUIINFO_END
 };
 
+gboolean row_selected_by_button_press_event;
+
 void
 die_cb (GnomeClient 	*client, 
 	gpointer 	data)
@@ -89,14 +93,14 @@ help_cb (GtkWidget 	*widget,
                 dialog = gtk_message_dialog_new (GTK_WINDOW (interface.main_window),
                         GTK_DIALOG_DESTROY_WITH_PARENT,
                         GTK_MESSAGE_ERROR,
-                        GTK_BUTTONS_CLOSE,
-                        _("There was an error displaying help: %s"),
+                        GTK_BUTTONS_OK,
                         error->message);
 
                 g_signal_connect (G_OBJECT (dialog),
                         "response",
                         G_CALLBACK (gtk_widget_destroy), NULL);
-			
+		
+		gtk_window_set_title (GTK_WINDOW (dialog), _("Can't Display Help"));	
                 gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
                 gtk_widget_show (dialog);
                 g_error_free (error);
@@ -235,7 +239,7 @@ constraint_entry_changed_cb (GtkWidget 	*widget,
 	} 
 		
 	look_in_folder_string = 
-		gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(interface.look_in_folder_entry), TRUE);
+		gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(interface.look_in_folder_entry), FALSE);
 
 	if (look_in_folder_string == NULL) {
 		gtk_widget_set_sensitive (interface.find_button, FALSE);
@@ -260,63 +264,167 @@ void
 open_file_cb (GtkWidget 	*widget, 
 	      gpointer 		data)
 {
-	gchar *file = NULL;
-	gchar *locale_file = NULL;
-	gchar *utf8_name = NULL;
-	gchar *utf8_path = NULL;
-	gboolean no_files_found = FALSE;
-	GtkListStore *store;
-	GtkTreeIter iter;
-	GtkTreeSelection *selection;
+	GList *list;
+	guint index;
+
+	if (gtk_tree_selection_count_selected_rows (GTK_TREE_SELECTION(interface.selection)) == 0) {
+		return;
+	}
 	
-		store = interface.model;
-		selection = interface.selection;
+	list = gtk_tree_selection_get_selected_rows (GTK_TREE_SELECTION(interface.selection),
+						     (GtkTreeModel **)&interface.model);
+	
+	if (g_list_length (list) > SILENT_WINDOW_OPEN_LIMIT) {
+	
+		GtkWidget *dialog;
+		gchar     *title;
+		gint      response;
+
+        	dialog = gtk_message_dialog_new (GTK_WINDOW (interface.main_window),
+                       		GTK_DIALOG_MODAL,
+                       		GTK_MESSAGE_QUESTION,
+                       		GTK_BUTTONS_NONE,
+                      		_("This will open %d separate files. Are you sure you want to do this?"),
+				g_list_length (list));
+
+		title = g_strdup_printf (_("Open %d Files?"), g_list_length (list));
+		gtk_window_set_title (GTK_WINDOW (dialog), title);
 		
-		if (!gtk_tree_selection_get_selected (selection, NULL, &iter))
-    			return;
+		gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+            		GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
+            		GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
+            		NULL);
 		
-		gtk_tree_model_get(GTK_TREE_MODEL(store),&iter,COLUMN_NAME, &utf8_name,
-							       COLUMN_PATH, &utf8_path,
-							       COLUMN_NO_FILES_FOUND, &no_files_found,
-								-1);
+		gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_REJECT);
+		
+		response = gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
+		
+		if (response == GTK_RESPONSE_REJECT) {
+			g_list_free (list);
+			return;
+		}
+		g_free (title);        
+	}
+		
+	for (index = 0; index < g_list_length(list); index++) {
+	
+		gboolean no_files_found = FALSE;
+		gchar *utf8_name;
+		gchar *utf8_path;
+		GtkTreeIter iter;
+			
+		gtk_tree_model_get_iter (GTK_TREE_MODEL(interface.model), &iter, 
+					 g_list_nth(list, index)->data);
+
+		gtk_tree_model_get (GTK_TREE_MODEL(interface.model), &iter,
+    			    COLUMN_NAME, &utf8_name,
+		    	    COLUMN_PATH, &utf8_path,
+		    	    COLUMN_NO_FILES_FOUND, &no_files_found,
+		   	    -1);		    
 
 		if (!no_files_found) {
+			gchar *file;
+			gchar *locale_file;
+				
 			file = g_build_filename (utf8_path, utf8_name, NULL);
-			locale_file = g_locale_from_utf8(file, -1, NULL, NULL, NULL);
+			locale_file = g_locale_from_utf8 (file, -1, NULL, NULL, NULL);
 			
-			if (is_nautilus_running ()) {
+			if (is_component_action_type (locale_file) 
+		    	    && is_nautilus_running ()) {
 				open_file_with_nautilus (locale_file);
 			}
 			else {
-				if (!open_file_with_application(locale_file))	
-					gnome_error_dialog_parented(_("No viewer available for this mime type."),
-							    	    GTK_WINDOW(interface.main_window));
-			}
+				if (open_file_with_application (locale_file) == FALSE) {
+				
+					if (launch_file (locale_file) == FALSE) {
+						GtkWidget *dialog;
 
+                				dialog = gtk_message_dialog_new (GTK_WINDOW (interface.main_window),
+                        				GTK_DIALOG_DESTROY_WITH_PARENT,
+                        				GTK_MESSAGE_ERROR,
+                        				GTK_BUTTONS_OK,
+                        				_("There is no installed viewer capable of displaying \"%s\"."),
+							file);
+
+                				g_signal_connect (G_OBJECT (dialog),
+                        				"response",
+                        				G_CALLBACK (gtk_widget_destroy), NULL);
+				
+       	         				gtk_window_set_title (GTK_WINDOW (dialog), _("Can't Display Location"));
+       	         				gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+	                			gtk_widget_show (dialog);
+					}
+				}
+			} 
 			g_free (file);
 			g_free (locale_file);
 		}
-		
-	g_free (utf8_name);
-	g_free (utf8_path);
-	
+		g_free (utf8_name);
+		g_free (utf8_path);
+	} 
+	g_list_free (list);
 }
 
 void
 open_folder_cb (GtkWidget 	*widget, 
 		gpointer 	data)
 {
-	GtkTreeSelection *selection;
-	GtkListStore *store;
-	GtkTreeIter iter;
-	gchar *folder_locale;
-	gchar *folder_utf8;
+	GList *list;
+	guint index;
 	
-	store = interface.model;
-	selection = interface.selection;
+	if (gtk_tree_selection_count_selected_rows (GTK_TREE_SELECTION(interface.selection)) == 0) {
+		return;
+	}
+	
+	list = gtk_tree_selection_get_selected_rows (GTK_TREE_SELECTION(interface.selection),
+						     (GtkTreeModel **)&interface.model);
+	
+	if (g_list_length (list) > SILENT_WINDOW_OPEN_LIMIT) {
+		GtkWidget *dialog;
+		gchar     *title;
+		gint      response;
+
+        	dialog = gtk_message_dialog_new (GTK_WINDOW (interface.main_window),
+                       		GTK_DIALOG_MODAL,
+                       		GTK_MESSAGE_QUESTION,
+                       		GTK_BUTTONS_NONE,
+                      		_("This will open %d separate folders. Are you sure you want to do this?"),
+				g_list_length (list));
+
+		title = g_strdup_printf (_("Open %d Folders?"), g_list_length (list));
+		gtk_window_set_title (GTK_WINDOW (dialog), title);
 		
-	if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {		
-		gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, COLUMN_PATH, &folder_utf8, -1);						
+		gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+            		GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
+            		GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
+            		NULL);
+		
+		gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_REJECT);
+		
+		response = gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
+		
+		if (response == GTK_RESPONSE_REJECT) {
+			g_list_free (list);
+			return;
+		}
+		g_free (title);      
+	}
+		
+	for (index = 0; index < g_list_length(list); index++) {
+		
+		gchar *folder_locale;
+		gchar *folder_utf8;
+		GtkTreeIter iter;
+		
+		gtk_tree_model_get_iter (GTK_TREE_MODEL(interface.model), &iter, 
+					 g_list_nth(list, index)->data);
+					 
+		gtk_tree_model_get (GTK_TREE_MODEL(interface.model), &iter,
+				    COLUMN_PATH, &folder_utf8,
+				    -1);
+			    							
 		folder_locale = g_filename_from_utf8 (folder_utf8, -1, NULL, NULL, NULL);
 		
 		if (is_nautilus_running ()) {
@@ -325,85 +433,99 @@ open_folder_cb (GtkWidget 	*widget,
 		else {
 			gnome_error_dialog_parented (_("The nautilus file manager is not running."),
 						       GTK_WINDOW(interface.main_window));
+			g_free (folder_locale);
+			g_free (folder_utf8);
+			g_list_free (list);
+			return;
 		}
 		g_free (folder_locale);
 		g_free (folder_utf8);
 	}
+	g_list_free (list);
 }
 
 gboolean
-click_file_cb 	     (GtkWidget 	*widget, 
-		      GdkEventButton 	*event, 
-		      gpointer 		data)
+file_button_press_event_cb (GtkWidget 		*widget, 
+			    GdkEventButton 	*event, 
+			    gpointer 		data)
+{
+	GtkTreePath *path;
+	row_selected_by_button_press_event = TRUE;
+
+	if (event->window != gtk_tree_view_get_bin_window (GTK_TREE_VIEW(interface.tree))) {
+		return FALSE;
+	}
+	
+	if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW(interface.tree), event->x, event->y,
+		&path, NULL, NULL, NULL)) {
+		
+		if ((event->button == 1 || event->button == 3)
+			&& gtk_tree_selection_path_is_selected (gtk_tree_view_get_selection (GTK_TREE_VIEW(interface.tree)), path)) {
+			
+			row_selected_by_button_press_event = FALSE;
+		}
+		
+		gtk_tree_path_free (path);
+	}
+	else {
+		gtk_tree_selection_unselect_all (gtk_tree_view_get_selection (GTK_TREE_VIEW(interface.tree)));
+	}
+	
+	return !(row_selected_by_button_press_event);
+}
+
+gboolean
+file_button_release_event_cb (GtkWidget 	*widget, 
+		      	      GdkEventButton 	*event, 
+		      	      gpointer 		data)
 {
 	gboolean no_files_found = FALSE;	
-	GtkTreeSelection *selection;
-	GtkTreePath *path;
 	GtkTreeIter iter;
 
-	if (! gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (interface.tree),
-					     event->x, event->y,
-					     &path, NULL, NULL, NULL)) {
-		return FALSE;
-	}
-		
-	if (! gtk_tree_model_get_iter (GTK_TREE_MODEL (interface.model), &iter, path)) {
-		gtk_tree_path_free (path);
-		return FALSE;
-	}
-	gtk_tree_path_free (path);
-
-	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (interface.tree));
-	if (selection == NULL) {
+	if (event->window != gtk_tree_view_get_bin_window (GTK_TREE_VIEW(interface.tree))) {
 		return FALSE;
 	}
 	
-	if (! gtk_tree_selection_iter_is_selected (selection, &iter)) {
-		gtk_tree_selection_unselect_all (selection);
-		gtk_tree_selection_select_iter (selection, &iter);
-	}
-
-	if (!gtk_tree_selection_get_selected (selection, NULL, &iter)) {
+	if (gtk_tree_selection_count_selected_rows (GTK_TREE_SELECTION(interface.selection)) == 0) {
 		return FALSE;
 	}
-			
-	if (event->type == GDK_2BUTTON_PRESS) {
-	
-		gchar *file = NULL;
-		gchar *locale_file = NULL;
-		gchar *utf8_name = NULL;
-		gchar *utf8_path = NULL;
 		
-		gtk_tree_model_get (GTK_TREE_MODEL(interface.model), &iter,
-			    	    COLUMN_NAME, &utf8_name,
-			    	    COLUMN_PATH, &utf8_path,
-			    	    COLUMN_NO_FILES_FOUND, &no_files_found,
-			   	    -1);
-		
-		if (!no_files_found) {
-			file = g_build_filename (utf8_path, utf8_name, NULL);
-			locale_file = g_locale_from_utf8(file, -1, NULL, NULL, NULL);
-			
-			if (is_nautilus_running ()) {
-				open_file_with_nautilus (locale_file);
+	if (event->button == 1) {
+
+		GtkTreePath *path;
+
+		if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW(interface.tree), event->x, event->y,
+			&path, NULL, NULL, NULL)) {
+			if ((event->state & GDK_SHIFT_MASK) || (event->state & GDK_CONTROL_MASK)) {
+				if (row_selected_by_button_press_event == TRUE) {
+					gtk_tree_selection_select_path (gtk_tree_view_get_selection (GTK_TREE_VIEW(interface.tree)), path);
+				}
+				else {
+					gtk_tree_selection_unselect_path (gtk_tree_view_get_selection (GTK_TREE_VIEW(interface.tree)), path);
+				}
 			}
 			else {
-				if (!open_file_with_application(locale_file))	
-					gnome_error_dialog_parented(_("No viewer available for this mime type."),
-							    	    GTK_WINDOW(interface.main_window));
+				gtk_tree_selection_unselect_all (gtk_tree_view_get_selection (GTK_TREE_VIEW(interface.tree)));
+				gtk_tree_selection_select_path (gtk_tree_view_get_selection (GTK_TREE_VIEW(interface.tree)), path);
 			}
 		}
-		g_free (file);
-		g_free (locale_file);
-		g_free (utf8_name);
-		g_free (utf8_path);
-	}
-	else if ((event->type == GDK_BUTTON_PRESS) && (event->button == 3)) {	
-		GtkWidget *popup;
+
+		gtk_tree_path_free (path);
+	}	
+	if (event->button == 3) {	
 		
+		GtkWidget *popup;
+		GList *list;
+		
+		list = gtk_tree_selection_get_selected_rows (GTK_TREE_SELECTION(interface.selection),
+							     (GtkTreeModel **)&interface.model);
+		
+		gtk_tree_model_get_iter (GTK_TREE_MODEL(interface.model), &iter, 
+					 g_list_first (list)->data);
+					 
 		gtk_tree_model_get (GTK_TREE_MODEL(interface.model), &iter,
 			    	    COLUMN_NO_FILES_FOUND, &no_files_found,
-			   	    -1);
+			   	    -1);		    
 				     
 		if (!no_files_found) {
 			GtkWidget *save_widget;
@@ -420,11 +542,108 @@ click_file_cb 	     (GtkWidget 	*widget,
 			
 			gnome_popup_menu_do_popup (GTK_WIDGET (popup), NULL, NULL, 
 						   (GdkEventButton *)event, data, NULL);
+						   
 		} 
+		g_list_free (list);
+	}	
+	return FALSE;
+}
+
+gboolean
+file_event_after_cb  (GtkWidget 	*widget, 
+		      GdkEventButton 	*event, 
+		      gpointer 		data)
+{	
+	GtkTreeIter 	 iter;
+
+	if (event->window != gtk_tree_view_get_bin_window (GTK_TREE_VIEW(interface.tree))) {
+		return FALSE;
+	}
+	
+	if (gtk_tree_selection_count_selected_rows (GTK_TREE_SELECTION(interface.selection)) == 0) {
+		return FALSE;
+	}
+		
+	if (event->type == GDK_2BUTTON_PRESS) {
+
+		gboolean no_files_found = FALSE;
+		gchar *utf8_name;
+		gchar *utf8_path;	 
+		GList *list;
+		
+		list = gtk_tree_selection_get_selected_rows (GTK_TREE_SELECTION(interface.selection),
+							     (GtkTreeModel **)&interface.model);
+		
+		gtk_tree_model_get_iter (GTK_TREE_MODEL(interface.model), &iter, 
+					 g_list_first (list)->data);
+					 
+		gtk_tree_model_get (GTK_TREE_MODEL(interface.model), &iter,
+    				    COLUMN_NAME, &utf8_name,
+			    	    COLUMN_PATH, &utf8_path,
+			    	    COLUMN_NO_FILES_FOUND, &no_files_found,
+			   	    -1);	
+		
+		if (!no_files_found) {
+			
+			gchar *file;
+			gchar *locale_file;
+			
+			file = g_build_filename (utf8_path, utf8_name, NULL);
+			locale_file = g_locale_from_utf8 (file, -1, NULL, NULL, NULL);
+			
+			if (is_component_action_type (locale_file) 
+			    && is_nautilus_running ()) {
+				open_file_with_nautilus (locale_file);
+			}
+			else {
+				if (open_file_with_application (locale_file) == FALSE) {
+				
+					if (launch_file (locale_file) == FALSE) {
+						GtkWidget *dialog;
+
+                				dialog = gtk_message_dialog_new (GTK_WINDOW (interface.main_window),
+                        				GTK_DIALOG_DESTROY_WITH_PARENT,
+                        				GTK_MESSAGE_ERROR,
+                        				GTK_BUTTONS_OK,
+                        				_("There is no installed viewer capable of displaying \"%s\"."),
+							file);
+
+                				g_signal_connect (G_OBJECT (dialog),
+                        				"response",
+                        				G_CALLBACK (gtk_widget_destroy), NULL);
+				
+       	         				gtk_window_set_title (GTK_WINDOW (dialog), _("Can't Display Location"));
+       	         				gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+	                			gtk_widget_show (dialog);
+					}
+				}
+			} 
+			g_free (file);
+			g_free (locale_file);
+		}
+		g_free (utf8_name);
+		g_free (utf8_path);
+		g_list_free (list);
 	}
 	return FALSE;
 }
 
+void  
+drag_begin_file_cb  (GtkWidget          *widget,
+		     GdkDragContext     *context,
+		     GtkSelectionData   *selection_data,
+		     guint               info,
+		     guint               time,
+		     gpointer            data)
+{	
+	if (gtk_tree_selection_count_selected_rows (GTK_TREE_SELECTION(interface.selection)) > 1) {
+		gtk_drag_set_icon_stock (context, GTK_STOCK_DND_MULTIPLE, 0, 0);
+	}
+	else {
+		gtk_drag_set_icon_stock (context, GTK_STOCK_DND, 0, 0);
+	}
+}
+
 void  
 drag_file_cb  (GtkWidget          *widget,
 	       GdkDragContext     *context,
@@ -432,49 +651,63 @@ drag_file_cb  (GtkWidget          *widget,
 	       guint               info,
 	       guint               time,
 	       gpointer            data)
-{
-	gchar 	 		*file = NULL;
-	gchar 	 		*locale_file = NULL;
-	gchar    		*url_file = NULL;
-	gchar 	 		*utf8_name = NULL;
-	gchar 	 		*utf8_path = NULL;
-	gboolean 		no_files_found = FALSE;
-	GtkTreeSelection 	*selection;
-	GtkListStore 		*store;
-	GtkTreeIter 		iter;
-	
-	store = interface.model;
-	selection = interface.selection;
-	
-	if (!gtk_tree_selection_get_selected (selection, NULL, &iter))
+{	
+	gchar    	*uri_list = NULL;
+	GList 		*list;
+	GtkTreeIter 	iter;
+	guint 		index;	
+
+	if (gtk_tree_selection_count_selected_rows (GTK_TREE_SELECTION(interface.selection)) == 0) {
 		return;
+	}
 	
-	gtk_tree_model_get(GTK_TREE_MODEL(store),&iter,COLUMN_NAME, &utf8_name,
-     		           COLUMN_PATH, &utf8_path,
-			   COLUMN_NO_FILES_FOUND, &no_files_found,
-		           -1);
+	list = gtk_tree_selection_get_selected_rows (GTK_TREE_SELECTION(interface.selection),
+						     (GtkTreeModel **)&interface.model);
+	
+	for (index = 0; index < g_list_length (list); index++) {
 			   	   
-	file = g_build_filename (utf8_path, utf8_name, NULL);
-	locale_file = g_locale_from_utf8 (file, -1, NULL, NULL, NULL);
+		gboolean   no_files_found = FALSE;
+		gchar	   *utf8_name;
+		gchar	   *utf8_path;
+		gchar	   *file;
+		
+		gtk_tree_model_get_iter (GTK_TREE_MODEL(interface.model), &iter, 
+					 g_list_nth(list, index)->data);
+					 
+		gtk_tree_model_get (GTK_TREE_MODEL(interface.model), &iter,
+    			    COLUMN_NAME, &utf8_name,
+		    	    COLUMN_PATH, &utf8_path,
+		    	    COLUMN_NO_FILES_FOUND, &no_files_found,
+		   	    -1);	
+			    
+		file = g_build_filename (utf8_path, utf8_name, NULL);
 
-	if (!no_files_found) {
-		url_file = g_strconcat ("file://", locale_file, NULL);	
-	} 
-	else {
-		url_file = g_strconcat (locale_file, NULL);
-	}	
-
-	g_free (locale_file);
-	g_free (file);
-
-	gtk_selection_data_set (selection_data, 
-				selection_data->target,
-				8, 
-				url_file, 
-				strlen (url_file));
-	g_free (utf8_name);
-	g_free (utf8_path);
-	g_free (url_file);
+		if (!no_files_found) {
+			gchar *tmp_uri = g_filename_to_uri (file, NULL, NULL);
+			
+			if (uri_list == NULL) {
+				uri_list = g_strdup (tmp_uri);
+			}
+			else{
+				uri_list = g_strconcat (uri_list, "\n", tmp_uri, NULL);
+			}
+			
+			gtk_selection_data_set (selection_data, 
+						selection_data->target,
+						8, 
+						uri_list, 
+						strlen (uri_list));
+			g_free (tmp_uri);
+		} 
+		else {
+			gtk_selection_data_set_text (selection_data, utf8_name, -1);
+		}	
+		g_free (utf8_name);
+		g_free (utf8_path);
+		g_free (file);
+	}
+	g_list_free (list);
+	g_free (uri_list);
 }
 
 void
@@ -501,7 +734,7 @@ show_file_selector_cb (GtkWidget 	*widget,
 
 	gtk_window_set_modal (GTK_WINDOW(interface.file_selector), TRUE);
 	gtk_window_set_transient_for (GTK_WINDOW(interface.file_selector), GTK_WINDOW(interface.main_window));
-	gtk_window_set_position (GTK_WINDOW (interface.file_selector), GTK_WIN_POS_MOUSE);
+	gtk_window_set_position (GTK_WINDOW (interface.file_selector), GTK_WIN_POS_CENTER_ON_PARENT);
 
 	gtk_widget_show (GTK_WIDGET(interface.file_selector));
 }
@@ -529,7 +762,7 @@ save_results_cb (GtkFileSelection *selector,
 			 0 /* flags */,
 			 GTK_MESSAGE_QUESTION,
 			 GTK_BUTTONS_YES_NO,
-			 _("File %s already exists. Overwrite?"),
+			 _("File \"%s\" already exists. Overwrite?"),
 			 interface.save_results_file);
 		gtk_window_set_transient_for (GTK_WINDOW(dialog), GTK_WINDOW (interface.main_window));
 		response = gtk_dialog_run (GTK_DIALOG (dialog));
@@ -564,7 +797,23 @@ save_results_cb (GtkFileSelection *selector,
 		fclose(fp);
 	} 
 	else {
-		gnome_app_error(GNOME_APP(interface.main_window), _("Cannot save the results file."));
+		GtkWidget *dialog;
+		gchar *error_msg = g_strdup_printf (_("Cannot save search results to the file \"%s\"."),
+						      interface.save_results_file);
+				
+		dialog = gtk_message_dialog_new (GTK_WINDOW (interface.main_window),
+				GTK_DIALOG_DESTROY_WITH_PARENT,
+				GTK_MESSAGE_ERROR,
+				GTK_BUTTONS_OK,
+				error_msg);
+
+		g_signal_connect (G_OBJECT (dialog),
+				"response",
+				G_CALLBACK (gtk_widget_destroy), NULL);
+					
+		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+		gtk_widget_show (dialog);
+		g_free (error_msg);
 	}
 }
 
@@ -601,20 +850,24 @@ key_press_cb (GtkWidget    	*widget,
 	}
 	else if (event->keyval == GDK_F10) {
 		if (event->state & GDK_SHIFT_MASK) {
-			GtkWidget *popup;
-			gchar *utf8_name = NULL;
-			gchar *utf8_path = NULL;
 			gboolean no_files_found = FALSE;
+			GtkWidget *popup;
 			GtkTreeIter iter;
+			GList *list;
+			
+			if (gtk_tree_selection_count_selected_rows (GTK_TREE_SELECTION(interface.selection)) == 0) {
+				return FALSE;
+			}
 		
-			if (!gtk_tree_selection_get_selected (interface.selection, NULL, &iter))
-    				return FALSE;
+			list = gtk_tree_selection_get_selected_rows (GTK_TREE_SELECTION(interface.selection),
+						     (GtkTreeModel **)&interface.model);
+			
+			gtk_tree_model_get_iter (GTK_TREE_MODEL(interface.model), &iter, 
+						 g_list_first(list)->data);
 		
 			gtk_tree_model_get (GTK_TREE_MODEL(interface.model), &iter,
-					    COLUMN_NAME, &utf8_name,
-					    COLUMN_PATH, &utf8_path,
-				    	COLUMN_NO_FILES_FOUND, &no_files_found,
-				    	-1);
+					    COLUMN_NO_FILES_FOUND, &no_files_found,
+				    	    -1);
 				    
 			if (!no_files_found) {
 				popup = gnome_popup_menu_new (popup_menu);
