@@ -57,19 +57,24 @@ struct _FindOptionTemplate {
 };
 	
 static FindOptionTemplate templates[] = {
-	{ SEARCH_CONSTRAINT_TEXT, "-exec grep -q '%s' {} \\;", N_("Contains the text"), FALSE },
+	{ SEARCH_CONSTRAINT_TEXT, "-exec grep -c '%s' {} \\;", N_("Contains the text"), FALSE },
+	{ SEARCH_CONSTRAINT_SEPARATOR, NULL, NULL, TRUE },
+	{ SEARCH_CONSTRAINT_TIME, "-mtime -%d", N_("Date modified less than (days)"), FALSE },
+	{ SEARCH_CONSTRAINT_TIME, "-mtime +%d", N_("Date modified more than (days)"), FALSE },
+	{ SEARCH_CONSTRAINT_SEPARATOR, NULL, NULL, TRUE },
+	{ SEARCH_CONSTRAINT_NUMBER, "-size +%uc", N_("Size at least (kilobytes)"), FALSE }, 
+	{ SEARCH_CONSTRAINT_NUMBER, "-size -%uc", N_("Size at most (kilobytes)"), FALSE },
+	{ SEARCH_CONSTRAINT_BOOL, "-size 0c \\( -type f -o -type d \\)", N_("File is empty"), FALSE },	
+	{ SEARCH_CONSTRAINT_SEPARATOR, NULL, NULL, TRUE },
 	{ SEARCH_CONSTRAINT_TEXT, "-user '%s'", N_("Owned by user"), FALSE },
 	{ SEARCH_CONSTRAINT_TEXT, "-group '%s'", N_("Owned by group"), FALSE },
 	{ SEARCH_CONSTRAINT_BOOL, "\\( -nouser -o -nogroup \\)", N_("Owner is unrecognized"), FALSE },
-	{ SEARCH_CONSTRAINT_TIME, "-mtime -%d", N_("Date modified before (days)"), FALSE },
-	{ SEARCH_CONSTRAINT_TIME, "-mtime +%d", N_("Date modified after (days)"), FALSE },	
-	{ SEARCH_CONSTRAINT_NUMBER, "-size -%dk", N_("Size is less than (kilobytes)"), FALSE },
-	{ SEARCH_CONSTRAINT_NUMBER, "-size +%dk", N_("Size is more than (kilobytes)"), FALSE }, 
-	{ SEARCH_CONSTRAINT_BOOL, "-size 0c \\( -type f -o -type d \\)", N_("File is empty"), FALSE },
+	{ SEARCH_CONSTRAINT_SEPARATOR, NULL, NULL, TRUE },
 	{ SEARCH_CONSTRAINT_TEXT, "'!' -name '%s'", N_("File is not named"), FALSE },
 	{ SEARCH_CONSTRAINT_TEXT, "-regex '%s'", N_("File matches regular expression"), FALSE }, 
+	{ SEARCH_CONSTRAINT_SEPARATOR, NULL, NULL, TRUE },
 	{ SEARCH_CONSTRAINT_BOOL, "-follow", N_("Follow symbolic links"), FALSE },
-	{ SEARCH_CONSTRAINT_BOOL, "-xdev", N_("Search other filesystems"), FALSE },
+	{ SEARCH_CONSTRAINT_BOOL, "-xdev", N_("Include other filesystems"), FALSE },
 	{ SEARCH_CONSTRAINT_END, NULL, NULL, FALSE}
 }; 
 
@@ -82,17 +87,22 @@ static GtkTargetEntry dnd_table[] = {
 static guint n_dnds = sizeof (dnd_table) / sizeof (dnd_table[0]);
 
 enum {
-	SEARCH_CONSTRAINT_CONTAINS_THE_TEXT, 
+	SEARCH_CONSTRAINT_CONTAINS_THE_TEXT,
+	SEARCH_CONSTRAINT_SEPARATOR_00, 
+	SEARCH_CONSTRAINT_DATE_MODIFIED_BEFORE,
+	SEARCH_CONSTRAINT_DATE_MODIFIED_AFTER,
+	SEARCH_CONSTRAINT_SEPARATOR_01,
+	SEARCH_CONSTRAINT_SIZE_IS_MORE_THAN,	
+	SEARCH_CONSTRAINT_SIZE_IS_LESS_THAN,
+	SEARCH_CONSTRAINT_FILE_IS_EMPTY,
+	SEARCH_CONSTRAINT_SEPARATOR_02,	
 	SEARCH_CONSTRAINT_OWNED_BY_USER, 
 	SEARCH_CONSTRAINT_OWNED_BY_GROUP,
 	SEARCH_CONSTRAINT_OWNER_IS_UNRECOGNIZED,
-	SEARCH_CONSTRAINT_DATE_MODIFIED_BEFORE,
-	SEARCH_CONSTRAINT_DATE_MODIFIED_AFTER,
-	SEARCH_CONSTRAINT_SIZE_IS_LESS_THAN,
-	SEARCH_CONSTRAINT_SIZE_IS_MORE_THAN,
-	SEARCH_CONSTRAINT_FILE_IS_EMPTY,
+	SEARCH_CONSTRAINT_SEPARATOR_03,	
 	SEARCH_CONSTRAINT_FILE_IS_NOT_NAMED,
 	SEARCH_CONSTRAINT_FILE_MATCHES_REGULAR_EXPRESSION,
+	SEARCH_CONSTRAINT_SEPARATOR_04,	
 	SEARCH_CONSTRAINT_FOLLOW_SYMBOLIC_LINKS,
 	SEARCH_CONSTRAINT_SEARCH_OTHER_FILESYSTEMS
 };
@@ -115,7 +125,7 @@ struct _PoptArgument {
 	gboolean	allmounts;
 	gchar 		*sortby;
 	gboolean 	descending;
-	gboolean 	autostart;
+	gboolean 	start;
 } PoptArgument;
 
 struct poptOption options[] = { 
@@ -127,26 +137,26 @@ struct poptOption options[] = {
 	  N_("Sort files by one of the following: name, folder, size, type, or date"), NULL},
   	{ "descending", '\0', POPT_ARG_NONE, &PoptArgument.descending, 0, 
 	  N_("Set sort order to descending, the default is ascending"), NULL},
-  	{ "autostart", '\0', POPT_ARG_NONE, &PoptArgument.autostart, 0, 
+  	{ "start", '\0', POPT_ARG_NONE, &PoptArgument.start, 0, 
 	  N_("Automatically start a search"), NULL},
   	{ "contains", '\0', POPT_ARG_STRING, &PoptArgument.contains, 0, 
 	  N_("Select the 'Contains the text' constraint"), NULL},
-	{ "user", '\0', POPT_ARG_STRING, &PoptArgument.user, 0, 
+  	{ "mtimeless", '\0', POPT_ARG_STRING, &PoptArgument.mtimeless, 0, 
+	  N_("Select the 'Date modified less than (days)' constraint"), NULL},
+  	{ "mtimemore", '\0', POPT_ARG_STRING, &PoptArgument.mtimemore, 0, 
+	  N_("Select the 'Date modified more than (days)' constraint"), NULL},
+	{ "sizeless", '\0', POPT_ARG_STRING, &PoptArgument.sizeless, 0, 
+	  N_("Select the 'Size at most (kilobytes)' constraint"), NULL},
+	{ "sizemore", '\0', POPT_ARG_STRING, &PoptArgument.sizemore, 0, 
+	  N_("Select the 'Size at least (kilobytes)' constraint"), NULL},
+	{ "empty", '\0', POPT_ARG_NONE, &PoptArgument.empty, 0, 
+	  N_("Select the 'File is empty' constraint"), NULL},
+  	{ "user", '\0', POPT_ARG_STRING, &PoptArgument.user, 0, 
 	  N_("Select the 'Owned by user' constraint"), NULL},
   	{ "group", '\0', POPT_ARG_STRING, &PoptArgument.group, 0, 
 	  N_("Select the 'Owned by group' constraint"), NULL},
   	{ "nouser", '\0', POPT_ARG_NONE, &PoptArgument.nouser, 0, 
 	  N_("Select the 'Owner is unrecognized' constraint"), NULL},
-  	{ "mtimeless", '\0', POPT_ARG_STRING, &PoptArgument.mtimeless, 0, 
-	  N_("Select the 'Date modified before (days)' constraint"), NULL},
-  	{ "mtimemore", '\0', POPT_ARG_STRING, &PoptArgument.mtimemore, 0, 
-	  N_("Select the 'Date modified after (days)' constraint"), NULL},
-  	{ "sizeless", '\0', POPT_ARG_STRING, &PoptArgument.sizeless, 0, 
-	  N_("Select the 'Size is less than (kilobytes)' constraint"), NULL},
-  	{ "sizemore", '\0', POPT_ARG_STRING, &PoptArgument.sizemore, 0, 
-	  N_("Select the 'Size is more than (kilobytes)' constraint"), NULL},
-  	{ "empty", '\0', POPT_ARG_NONE, &PoptArgument.empty, 0, 
-	  N_("Select the 'File is empty' constraint"), NULL},
   	{ "notnamed", '\0', POPT_ARG_STRING, &PoptArgument.notnamed, 0, 
 	  N_("Select the 'File is not named' constraint"), NULL},
   	{ "regex", '\0', POPT_ARG_STRING, &PoptArgument.regex, 0, 
@@ -154,7 +164,7 @@ struct poptOption options[] = {
   	{ "follow", '\0', POPT_ARG_NONE, &PoptArgument.follow, 0, 
 	  N_("Select the 'Follow symbolic links' constraint"), NULL},
   	{ "allmounts", '\0', POPT_ARG_NONE, &PoptArgument.allmounts, 0, 
-	  N_("Select the 'Search other filesystems' constraint"), NULL},
+	  N_("Select the 'Include other filesystems' constraint"), NULL},
   	{ NULL,'\0', 0, NULL, 0, NULL, NULL}
 };
 
@@ -174,15 +184,19 @@ build_search_command (void)
 		file_is_named_locale = NULL;
 	} 
 	else {
-		file_is_named_locale = g_filename_from_utf8 (file_is_named_utf8, -1, NULL, NULL, NULL);
+		file_is_named_locale = g_locale_from_utf8 (file_is_named_utf8, -1, NULL, NULL, NULL);
+		gnome_entry_prepend_history (GNOME_ENTRY(interface.file_is_named_entry), TRUE, file_is_named_utf8);
 	}
 	
-	look_in_folder_utf8 = gnome_file_entry_get_full_path (GNOME_FILE_ENTRY(interface.look_in_folder_entry), TRUE);
-	look_in_folder_locale = g_filename_from_utf8 (look_in_folder_utf8, -1, NULL, NULL, NULL);
+	look_in_folder_utf8 = gnome_file_entry_get_full_path (GNOME_FILE_ENTRY(interface.look_in_folder_entry), FALSE);
+	look_in_folder_locale = g_locale_from_utf8 (look_in_folder_utf8, -1, NULL, NULL, NULL);
+	gnome_entry_prepend_history (GNOME_ENTRY(gnome_file_entry_gnome_entry (GNOME_FILE_ENTRY(interface.look_in_folder_entry))), 
+				     TRUE, look_in_folder_utf8);
 	
 	if (!file_extension_is (look_in_folder_locale, G_DIR_SEPARATOR_S)) {
 		look_in_folder_locale = g_strconcat (look_in_folder_locale, G_DIR_SEPARATOR_S, NULL);
 	}
+	search_command.look_in_folder = g_strdup(look_in_folder_locale);
 	
 	command = g_string_new ("");
 	
@@ -192,7 +206,7 @@ build_search_command (void)
 		
 		locate = g_find_program_in_path ("locate");
 		file_is_named_escaped = escape_single_quotes (file_is_named_locale);
-		search_command.file_is_named_pattern = g_strdup(file_is_named_locale);
+		search_command.file_is_named_pattern = g_strdup(file_is_named_utf8);
 		
 		if ((locate != NULL) && (is_path_in_home_folder (look_in_folder_locale) != TRUE)) {
 			g_string_append_printf (command, "%s '%s*%s'", 
@@ -213,7 +227,7 @@ build_search_command (void)
 		
 		search_command.regex_matching_enabled = FALSE;
 		file_is_named_escaped = escape_single_quotes (file_is_named_locale);
-		search_command.file_is_named_pattern = g_strdup(file_is_named_locale);
+		search_command.file_is_named_pattern = g_strdup(file_is_named_utf8);
 		
 		if (file_is_named_escaped == NULL) {
 			g_string_append_printf (command, "find \"%s\" '!' -type d '!' -type p -name '%s' ", 
@@ -273,7 +287,7 @@ build_search_command (void)
 			case SEARCH_CONSTRAINT_NUMBER:
 				g_string_append_printf (command,
 					  		templates[constraint->constraint_id].option,
-					  		constraint->data.number);
+					  		(constraint->data.number * 1024));
 				g_string_append_c (command, ' ');
 				break;
 			case SEARCH_CONSTRAINT_TIME:
@@ -305,11 +319,11 @@ add_file_to_search_results (const gchar 	*file,
 			    GtkListStore 	*store, 
 			    GtkTreeIter 	*iter)
 {					
-	const gchar *mime_type = gnome_vfs_mime_type_from_name (file);
+	gchar *mime_type = gnome_vfs_get_mime_type (file);
 	gchar *description = get_file_type_with_mime_type (file, mime_type);
 	gchar *icon_path = get_file_icon_with_mime_type (file, mime_type);
-	GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file (icon_path, NULL);
 	GnomeVFSFileInfo *vfs_file_info = gnome_vfs_file_info_new ();
+	GdkPixbuf *pixbuf = NULL;
 	gchar *readable_size, *readable_date, *date;
 	gchar *utf8_base_name, *utf8_dir_name;
 	gchar *base_name, *dir_name;
@@ -318,6 +332,10 @@ add_file_to_search_results (const gchar 	*file,
 		gtk_tree_view_set_headers_visible (GTK_TREE_VIEW(interface.tree), TRUE);
 	}
 	
+	if (icon_path != NULL) {
+		pixbuf = gdk_pixbuf_new_from_file (icon_path, NULL);
+	}
+	
 	if (pixbuf != NULL) {
 		GdkPixbuf *scaled;
 		int        new_w, new_h;
@@ -365,10 +383,14 @@ add_file_to_search_results (const gchar 	*file,
 			    COLUMN_NO_FILES_FOUND, FALSE,
 			    -1);
 
+	if (pixbuf != NULL) {
+		g_object_unref (G_OBJECT(pixbuf));
+	}
+
 	gnome_vfs_file_info_unref (vfs_file_info);
-	g_object_unref (G_OBJECT(pixbuf));
 	g_free (base_name);
 	g_free (dir_name);
+	g_free (mime_type);
 	g_free (utf8_base_name);
 	g_free (utf8_dir_name);
 	g_free (icon_path); 
@@ -383,6 +405,7 @@ add_no_files_found_message (GtkListStore 	*store,
 {
 	/* When the list is empty append a 'No Files Found.' message. */
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW(interface.tree), FALSE);
+	gtk_tree_view_columns_autosize (GTK_TREE_VIEW(interface.tree));
 	gtk_list_store_append (GTK_LIST_STORE(store), iter); 
 	gtk_list_store_set (GTK_LIST_STORE(store), iter,
 		    	    COLUMN_ICON, NULL, 
@@ -402,6 +425,7 @@ make_list_of_templates (void)
 {
 	GtkWidget *menu;
 	GtkWidget *menuitem;
+	GtkWidget *separator;
 	GSList    *group = NULL;
 	gint i;
 
@@ -409,14 +433,24 @@ make_list_of_templates (void)
 
 	for(i=0; templates[i].type != SEARCH_CONSTRAINT_END; i++) {
 		
-		menuitem = gtk_radio_menu_item_new_with_label (group,
-							       _(templates[i].desc));
-		g_signal_connect (G_OBJECT(menuitem), "toggled",
-				  G_CALLBACK(constraint_menu_toggled_cb),
-		        	  (gpointer)(long)i);
-		group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM(menuitem));
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-		gtk_widget_show (menuitem);
+		if (templates[i].type == SEARCH_CONSTRAINT_SEPARATOR) {
+			menuitem = gtk_menu_item_new ();
+			separator = gtk_hseparator_new ();
+			gtk_container_add (GTK_CONTAINER (menuitem), separator);	
+			gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+			gtk_widget_show (separator);
+			gtk_widget_show (menuitem);
+		} 
+		else {
+			menuitem = gtk_radio_menu_item_new_with_label (group,
+							      	_(templates[i].desc));
+			g_signal_connect (G_OBJECT(menuitem), "toggled",
+					  G_CALLBACK(constraint_menu_toggled_cb),
+		        		  (gpointer)(long)i);
+			group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (menuitem));
+			gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+			gtk_widget_show (menuitem);
+		}
 		
 		if (templates[i].is_selected == TRUE) {
 			gtk_widget_set_sensitive (menuitem, FALSE);
@@ -481,9 +515,11 @@ set_constraint_selected_state (gint 		constraint_id,
 		if (templates[index].is_selected == FALSE) {
 			gtk_option_menu_set_history (GTK_OPTION_MENU(interface.constraint_menu), index);
 			interface.selected_constraint = (long)index;
-			break;
+			gtk_widget_set_sensitive (interface.add_button, TRUE);
+			return;
 		}
 	}
+	gtk_widget_set_sensitive (interface.add_button, FALSE);
 }
 
 static void
@@ -655,7 +691,7 @@ handle_popt_args (void)
 							      GTK_SORT_ASCENDING);
 		}
 	}
-	if (PoptArgument.autostart == TRUE) {
+	if (PoptArgument.start == TRUE) {
 		click_find_cb (interface.find_button, NULL);
 	}
 }
@@ -680,7 +716,7 @@ handle_search_command_stdout_io (GIOChannel 	*ioc,
 
 		do {
 			gint  status;
-			gchar *locale = NULL;
+			gchar *utf8 = NULL;
 			gchar *filename = NULL;
 			
 			if (search_data->running != RUNNING) {
@@ -712,32 +748,35 @@ handle_search_command_stdout_io (GIOChannel 	*ioc,
 				continue;
 			}
 			
-			locale = g_locale_to_utf8 (string->str, -1, NULL, NULL, NULL);
-			if (locale == NULL) {
+			utf8 = g_locale_to_utf8 (string->str, -1, NULL, NULL, NULL);
+			if (utf8 == NULL) {
 				continue;
 			}
 			
-			filename = g_path_get_basename (locale);
+			if (strncmp (string->str, search_data->look_in_folder, strlen (search_data->look_in_folder)) == 0) { 
 			
-			if (fnmatch (search_data->file_is_named_pattern, filename, FNM_NOESCAPE) != FNM_NOMATCH) {
-				if (search_data->show_hidden_files == TRUE) {
-					if (search_data->regex_matching_enabled == FALSE) {
-						add_file_to_search_results (string->str, interface.model, &interface.iter);
-					} 
-					else if (compare_regex (search_data->regex_string, filename) == TRUE) {
-						add_file_to_search_results (string->str, interface.model, &interface.iter);
+				filename = g_path_get_basename (utf8);
+			
+				if (fnmatch (search_data->file_is_named_pattern, filename, FNM_NOESCAPE) != FNM_NOMATCH) {
+					if (search_data->show_hidden_files == TRUE) {
+						if (search_data->regex_matching_enabled == FALSE) {
+							add_file_to_search_results (string->str, interface.model, &interface.iter);
+						} 
+						else if (compare_regex (search_data->regex_string, filename) == TRUE) {
+							add_file_to_search_results (string->str, interface.model, &interface.iter);
+						}
 					}
-				}
-				else if (is_path_hidden (string->str) == FALSE) {
-					if (search_data->regex_matching_enabled == FALSE) {
-						add_file_to_search_results (string->str, interface.model, &interface.iter);
-					} 
-					else if (compare_regex (search_data->regex_string, filename) == TRUE) {
-						add_file_to_search_results (string->str, interface.model, &interface.iter);
+					else if (is_path_hidden (string->str) == FALSE) {
+						if (search_data->regex_matching_enabled == FALSE) {
+							add_file_to_search_results (string->str, interface.model, &interface.iter);
+						} 
+						else if (compare_regex (search_data->regex_string, filename) == TRUE) {
+							add_file_to_search_results (string->str, interface.model, &interface.iter);
+						}
 					}
 				}
 			}
-			g_free (locale);
+			g_free (utf8);
 			g_free (filename);
 			
 			gtk_tree_view_get_visible_rect (GTK_TREE_VIEW(interface.tree), &prior_rect);
@@ -756,7 +795,7 @@ handle_search_command_stdout_io (GIOChannel 	*ioc,
 				}
 			}	
 			
-		} while (g_io_channel_get_buffer_condition(ioc) == G_IO_IN);
+		} while (g_io_channel_get_buffer_condition (ioc) == G_IO_IN);
 		
 		g_string_free (string, TRUE);
 	}
@@ -811,60 +850,103 @@ static gboolean
 handle_search_command_stderr_io (GIOChannel 	*ioc, 
 				 GIOCondition 	condition, 
 				 gpointer 	data) 
-{
-	gboolean   truncate_error_msgs = FALSE;
-	gchar      *locale = NULL;
-	GError     *error = NULL;
-	GtkWidget  *dialog;
-	GString    *error_msgs;
-	GString    *string;
+{	
+	struct _SearchStruct *search_data = data;
+	static GString       *error_msgs = NULL;
+	static gboolean      truncate_error_msgs = FALSE;
 	
-	error_msgs = g_string_new (NULL);
-	string = g_string_new (NULL);
-
-	while (ioc->is_readable != TRUE);
-
-	while (g_io_channel_read_line_string (ioc, string, NULL, &error) == G_IO_STATUS_NORMAL) {
+	if ((condition == G_IO_IN) || (condition == G_IO_IN + G_IO_HUP)) { 
 	
-		if (truncate_error_msgs == FALSE) {
-			if ((strstr (string->str, "ermission denied") == NULL) &&
-			    (strstr (string->str, "No such file or directory") == NULL) &&
-			    (strncmp (string->str, "grep: ", 6) != 0) &&
-			    (strcmp (string->str, "find: ") != 0)){
-				locale = g_locale_to_utf8 (string->str, -1, NULL, NULL, NULL);
-				error_msgs = g_string_append (error_msgs, locale); 
-				truncate_error_msgs = limit_string_to_x_lines (error_msgs, 20);
+		GString         *string;
+		GError          *error = NULL;
+		gchar           *utf8 = NULL;
+		
+		string = g_string_new (NULL);
+		
+		if (error_msgs == NULL) {
+			error_msgs = g_string_new (NULL);
+		}
+
+		while (ioc->is_readable != TRUE);
+
+		do {
+			gint status;
+		
+			do {
+				status = g_io_channel_read_line_string (ioc, string, NULL, &error);
+	
+				while (gtk_events_pending ()) {
+					if (search_data->running == MAKE_IT_QUIT) {
+						break;
+					}
+					gtk_main_iteration ();
+				}
+					
+			} while (status == G_IO_STATUS_AGAIN);
+			
+			if (status != G_IO_STATUS_NORMAL) {
+				if (error != NULL) {
+					g_warning ("handle_search_command_stderr_io(): %s", error->message);
+					g_error_free (error);
+				}
+				continue;
+			}
+			
+			if (truncate_error_msgs == FALSE) {
+				if ((strstr (string->str, "ermission denied") == NULL) &&
+			   	    (strstr (string->str, "No such file or directory") == NULL) &&
+			   	    (strncmp (string->str, "grep: ", 6) != 0) &&
+			 	    (strcmp (string->str, "find: ") != 0)) { 
+					utf8 = g_locale_to_utf8 (string->str, -1, NULL, NULL, NULL);
+					error_msgs = g_string_append (error_msgs, utf8); 
+					truncate_error_msgs = limit_string_to_x_lines (error_msgs, 20);
+				}
 			}
-		}
 		
-		while (gtk_events_pending ()) {
-			gtk_main_iteration ();
-		}
+		} while (g_io_channel_get_buffer_condition (ioc) == G_IO_IN);
+		
+		g_string_free (string, TRUE);
+		g_free (utf8);
 	}
 	
-	if (error != NULL) {
-		g_warning ("handle_search_command_stderr_io(): %s", error->message);
-		g_error_free (error);
-	}
+	if (condition != G_IO_IN) { 
 	
-	if (error_msgs->len > 0) {	
-		error_msgs = g_string_prepend (error_msgs, 
-			     _("While searching the following errors were reported.\n\n"));
-				
-		if (truncate_error_msgs == TRUE) {
-			error_msgs = g_string_append (error_msgs, 
-				     _("\n... Too many errors to display ..."));
-		}
-		
-		gnome_app_error (GNOME_APP (interface.main_window), error_msgs->str);	
-	}
+		if (error_msgs != NULL) {
 
-	g_io_channel_shutdown(ioc, TRUE, NULL);
-	g_string_free (error_msgs, TRUE);	
-	g_string_free (string, TRUE);
-	g_free (locale);
+			if (error_msgs->len > 0) {	
+				
+				GtkWidget *dialog;
+				
+				error_msgs = g_string_prepend (error_msgs, 
+				     _("While searching the following errors were reported.\n\n"));
+				
+				if (truncate_error_msgs == TRUE) {
+					error_msgs = g_string_append (error_msgs, 
+				     		_("\n... Too many errors to display ..."));
+				}
+		
+				dialog = gtk_message_dialog_new (GTK_WINDOW (interface.main_window),
+					GTK_DIALOG_DESTROY_WITH_PARENT,
+					GTK_MESSAGE_ERROR,
+					GTK_BUTTONS_OK,
+					error_msgs->str);
+
+				g_signal_connect (G_OBJECT (dialog),
+					"response",
+					G_CALLBACK (gtk_widget_destroy), NULL);
+		
+				gtk_window_set_title (GTK_WINDOW (dialog), _("Search Errors"));	
+				gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+				gtk_widget_show (dialog);
 	
-	return FALSE;
+			}
+			truncate_error_msgs = FALSE;
+			g_string_truncate (error_msgs, 0);
+		}
+		g_io_channel_shutdown(ioc, TRUE, NULL);
+		return FALSE;
+	}
+	return TRUE;
 }
 
 void
@@ -885,10 +967,13 @@ spawn_search_command (gchar *command)
 		gnome_appbar_push (GNOME_APPBAR (interface.status_bar), "");
 		gtk_timeout_remove (search_command.timeout);
 		
-		dialog = gtk_message_dialog_new(NULL, 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-						_("Sorry, could not start the search.\n%s\n"), 
+		dialog = gtk_message_dialog_new(GTK_WINDOW (interface.main_window), 
+						GTK_DIALOG_DESTROY_WITH_PARENT, 
+						GTK_MESSAGE_ERROR, 
+						GTK_BUTTONS_OK,
 						error->message);
-		gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (interface.main_window));
+						
+		gtk_window_set_title (GTK_WINDOW (dialog), _("Search for Files Error"));
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (dialog);
 		g_error_free (error);
@@ -906,10 +991,13 @@ spawn_search_command (gchar *command)
 		gnome_appbar_push (GNOME_APPBAR (interface.status_bar), "");
 		gtk_timeout_remove (search_command.timeout);
 		
-		dialog = gtk_message_dialog_new(NULL, 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-						_("Sorry, could not start the search.\n%s\n"), 
+		dialog = gtk_message_dialog_new(GTK_WINDOW (interface.main_window), 
+						GTK_DIALOG_DESTROY_WITH_PARENT, 
+						GTK_MESSAGE_ERROR, 
+						GTK_BUTTONS_OK,
 						error->message);
-		gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (interface.main_window));
+
+		gtk_window_set_title (GTK_WINDOW (dialog), _("Search for Files Error"));
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (dialog);
 		g_error_free (error);
@@ -935,11 +1023,12 @@ spawn_search_command (gchar *command)
 	g_io_channel_set_encoding (ioc_stderr, NULL, NULL);
 	
 	g_io_channel_set_flags (ioc_stdout, G_IO_FLAG_NONBLOCK, NULL);
+	g_io_channel_set_flags (ioc_stderr, G_IO_FLAG_NONBLOCK, NULL);
 	
 	g_io_add_watch (ioc_stdout, G_IO_IN | G_IO_HUP,
 			handle_search_command_stdout_io, &search_command);	
-	g_io_add_watch (ioc_stderr, G_IO_HUP,
-			handle_search_command_stderr_io, NULL);
+	g_io_add_watch (ioc_stderr, G_IO_IN | G_IO_HUP,
+			handle_search_command_stderr_io, &search_command);
 
 	g_io_channel_unref (ioc_stdout);
 	g_io_channel_unref (ioc_stderr);
@@ -961,7 +1050,7 @@ create_constraint_box (SearchConstraint *opt, gchar *value)
 			gchar *desc = g_strconcat (_(templates[opt->constraint_id].desc), ".", NULL);
 			label = gtk_label_new (desc);
 			g_free (desc);
-			gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, GNOME_PAD);
+			gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, GNOME_PAD_SMALL);
 		}
 		break;
 	case SEARCH_CONSTRAINT_TEXT:
@@ -972,10 +1061,21 @@ create_constraint_box (SearchConstraint *opt, gchar *value)
 			label = gtk_label_new (desc);
 			g_free (desc);
 		}
-		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, GNOME_PAD);
+		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, GNOME_PAD_SMALL);
+		
+		if (templates[opt->constraint_id].type == SEARCH_CONSTRAINT_TEXT) {
+			entry = gtk_entry_new();
+			if (value != NULL) {
+				gtk_entry_set_text (GTK_ENTRY(entry), value);
+			}
+		}
+		else {
+			entry = gtk_spin_button_new_with_range (0, 999999999, 1);
+			if (value != NULL) {
+				gtk_spin_button_set_value (GTK_SPIN_BUTTON(entry), atoi(value));
+			}
+		}
 		
-		entry = gtk_entry_new();
-
 		if (interface.is_gail_loaded)
 		{
 			add_atk_namedesc (GTK_WIDGET(entry), _("Search Rule Value Entry"), 
@@ -996,11 +1096,7 @@ create_constraint_box (SearchConstraint *opt, gchar *value)
 				  NULL);
 				  
 		/* add label and text field */
-		gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
-		
-		if (value != NULL) {
-			gtk_entry_set_text (GTK_ENTRY(entry), value);
-		}
+		gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, GNOME_PAD_SMALL);
 		
 		break;
 	default:
@@ -1014,7 +1110,7 @@ create_constraint_box (SearchConstraint *opt, gchar *value)
 	g_signal_connect(G_OBJECT(button), "clicked",
 			 G_CALLBACK(remove_constraint_cb), opt);
         gtk_size_group_add_widget (interface.constraint_size_group, button);
-	gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, GNOME_PAD_SMALL);
+	gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
 	
 	if (interface.is_gail_loaded) {
 		gchar *desc = g_strdup_printf (_("Click to Remove the '%s' Rule"), 
@@ -1061,10 +1157,9 @@ create_additional_constraint_section (void)
 	GtkWidget *vbox1; 
 	GtkWidget *vbox2;
 	GtkWidget *label;
-	GtkWidget *button;
 
-	vbox1 = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
-	gtk_container_set_border_width (GTK_CONTAINER(vbox1), GNOME_PAD_SMALL);
+	vbox1 = gtk_vbox_new (FALSE, 0);
+	gtk_container_set_border_width (GTK_CONTAINER(vbox1), 0);
 
 	interface.constraint = gtk_vbox_new (FALSE, 0); 
 	vbox2 = gtk_vbox_new (FALSE, 0);
@@ -1072,13 +1167,13 @@ create_additional_constraint_section (void)
 	
 	gtk_box_pack_end (GTK_BOX(vbox2), hbox, FALSE, FALSE, 0);
 	
-	label = gtk_label_new_with_mnemonic (_("A_vailable search constraints:"));
-	gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, GNOME_PAD);
+	label = gtk_label_new_with_mnemonic (_("A_vailable options:"));
+	gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, GNOME_PAD_SMALL);
 	
 	interface.constraint_menu = gtk_option_menu_new ();
 	gtk_label_set_mnemonic_widget (GTK_LABEL(label), GTK_WIDGET(interface.constraint_menu));
 	gtk_option_menu_set_menu (GTK_OPTION_MENU(interface.constraint_menu), make_list_of_templates());
-	gtk_box_pack_start (GTK_BOX(hbox), interface.constraint_menu, TRUE, TRUE, GNOME_PAD_SMALL);
+	gtk_box_pack_start (GTK_BOX(hbox), interface.constraint_menu, TRUE, TRUE, GNOME_PAD);
 
 	if (interface.is_gail_loaded)
 	{
@@ -1087,17 +1182,17 @@ create_additional_constraint_section (void)
 		add_atk_relation (interface.constraint_menu, GTK_WIDGET(label), ATK_RELATION_LABELLED_BY);
 	}
 
-	button = gtk_button_new_from_stock (GTK_STOCK_ADD);
+	interface.add_button = gtk_button_new_from_stock (GTK_STOCK_ADD);
 	interface.constraint_size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
-	gtk_widget_set_size_request (GTK_WIDGET(button), 78, -1);
-	gtk_size_group_add_widget (interface.constraint_size_group, button);
+	gtk_size_group_add_widget (interface.constraint_size_group, interface.add_button);
+	gtk_size_group_add_widget (interface.constraint_size_group, gtk_button_new_from_stock (GTK_STOCK_REMOVE));
 	
-	g_signal_connect (G_OBJECT(button),"clicked",
+	g_signal_connect (G_OBJECT(interface.add_button),"clicked",
 			  G_CALLBACK(add_constraint_cb),NULL);
 	
-	gtk_box_pack_end (GTK_BOX(hbox), button, FALSE, FALSE, GNOME_PAD_SMALL); 
+	gtk_box_pack_end (GTK_BOX(hbox), interface.add_button, FALSE, FALSE, 0); 
 		
-	gtk_box_pack_start (GTK_BOX(vbox1), interface.constraint, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX(vbox1), interface.constraint, TRUE, TRUE, GNOME_PAD_SMALL);
 	
 	gtk_container_add (GTK_CONTAINER(vbox1), GTK_WIDGET(vbox2));
 	
@@ -1123,7 +1218,7 @@ create_search_results_section (void)
 	interface.results = gtk_scrolled_window_new (NULL, NULL);
 	gtk_widget_set_sensitive (GTK_WIDGET(interface.results), FALSE); 
 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW(interface.results), GTK_SHADOW_IN);	
-	gtk_container_set_border_width (GTK_CONTAINER(interface.results), GNOME_PAD_SMALL);
+	gtk_container_set_border_width (GTK_CONTAINER(interface.results), 0);
 	gtk_widget_set_usize (interface.results, 530, 200); 
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(interface.results),
                                         GTK_POLICY_AUTOMATIC,
@@ -1148,19 +1243,37 @@ create_search_results_section (void)
 	
 	interface.selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(interface.tree));
 	
+	gtk_tree_selection_set_mode (GTK_TREE_SELECTION(interface.selection),
+				     GTK_SELECTION_MULTIPLE);
+				     
 	gtk_drag_source_set (interface.tree, 
 			     GDK_BUTTON1_MASK,
 			     dnd_table, n_dnds, 
-			     GDK_ACTION_COPY);	
+			     GDK_ACTION_COPY | GDK_ACTION_MOVE);	
 
 	g_signal_connect (G_OBJECT (interface.tree), 
 			  "drag_data_get",
 			  G_CALLBACK (drag_file_cb), 
 			  NULL);
 			  
+	g_signal_connect (G_OBJECT (interface.tree),
+			  "drag_begin",
+			  G_CALLBACK (drag_begin_file_cb),
+			  NULL);	  
+			  
+	g_signal_connect (G_OBJECT(interface.tree), 
+			  "event_after",
+		          G_CALLBACK(file_event_after_cb), 
+			  NULL);
+			  
+	g_signal_connect (G_OBJECT(interface.tree), 
+			  "button_release_event",
+		          G_CALLBACK(file_button_release_event_cb), 
+			  NULL);
+			  		  
 	g_signal_connect (G_OBJECT(interface.tree), 
 			  "button_press_event",
-		          G_CALLBACK(click_file_cb), 
+		          G_CALLBACK(file_button_press_event_cb), 
 			  NULL);		   
 
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), interface.tree);
@@ -1182,9 +1295,7 @@ create_search_results_section (void)
         gtk_tree_view_column_pack_start (column, renderer, TRUE);
         gtk_tree_view_column_set_attributes (column, renderer,
                                              "text", COLUMN_NAME,
-					     NULL);
-					     
-	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
+					     NULL);				     
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 	gtk_tree_view_column_set_resizable (column, TRUE);				     
 	gtk_tree_view_column_set_sort_column_id (column, COLUMN_NAME); 
@@ -1195,7 +1306,6 @@ create_search_results_section (void)
 	column = gtk_tree_view_column_new_with_attributes (_("Folder"), renderer,
 							   "text", COLUMN_PATH,
 							   NULL);
-	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 	gtk_tree_view_column_set_resizable (column, TRUE);
 	gtk_tree_view_column_set_sort_column_id (column, COLUMN_PATH); 
@@ -1206,8 +1316,7 @@ create_search_results_section (void)
 	g_object_set( renderer, "xalign", 1.0, NULL);
 	column = gtk_tree_view_column_new_with_attributes (_("Size"), renderer,
 							   "text", COLUMN_READABLE_SIZE,
-							   NULL);						   
-	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
+							   NULL);
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 	gtk_tree_view_column_set_resizable (column, TRUE);
 	gtk_tree_view_column_set_sort_column_id (column, COLUMN_SIZE);
@@ -1218,7 +1327,6 @@ create_search_results_section (void)
 	column = gtk_tree_view_column_new_with_attributes (_("Type"), renderer,
 							   "text", COLUMN_TYPE,
 							   NULL);
-	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 	gtk_tree_view_column_set_resizable (column, TRUE);
 	gtk_tree_view_column_set_sort_column_id (column, COLUMN_TYPE);
@@ -1229,7 +1337,6 @@ create_search_results_section (void)
 	column = gtk_tree_view_column_new_with_attributes (_("Date Modified"), renderer,
 							   "text", COLUMN_READABLE_DATE,
 							   NULL);
-	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 	gtk_tree_view_column_set_resizable (column, TRUE);
 	gtk_tree_view_column_set_sort_column_id (column, COLUMN_DATE);
@@ -1247,7 +1354,6 @@ create_main_window (void)
 	GtkWidget 	*entry;
 	GtkWidget 	*folder_entry;
 	GtkWidget 	*button;
-	GtkWidget 	*hsep;
 	GtkWidget 	*table;
 	GtkWidget 	*results;
 	GtkWidget 	*window;
@@ -1327,7 +1433,8 @@ create_main_window (void)
 	gtk_entry_set_text (GTK_ENTRY(entry), string);
 	g_free (string);
 	
-	interface.disclosure = cddb_disclosure_new (_("Additional Constraints"), _("Additional Constraints"));
+	interface.disclosure = cddb_disclosure_new (_("Additional Options"),
+						    _("Additional Options"));
 	gtk_box_pack_start (GTK_BOX(window), interface.disclosure, FALSE, FALSE, 0);
 	
 	interface.additional_constraints = create_additional_constraint_section ();
@@ -1335,8 +1442,8 @@ create_main_window (void)
 
 	gtk_box_pack_start (GTK_BOX(window), GTK_WIDGET(interface.additional_constraints), FALSE, FALSE, 0);
 	
-	hsep = gtk_hseparator_new ();
-	gtk_box_pack_start (GTK_BOX(window),hsep,FALSE,FALSE,0);
+	results = create_search_results_section ();
+	gtk_box_pack_start (GTK_BOX(window), results, TRUE, TRUE, GNOME_PAD_SMALL);
 	
 	hbox = gtk_hbutton_box_new ();
 	gtk_button_box_set_layout (GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_END);
@@ -1373,9 +1480,6 @@ create_main_window (void)
 		add_atk_namedesc (GTK_WIDGET(interface.stop_button), NULL, _("Click to Stop the search"));
 	}
 
-	results = create_search_results_section ();
-	gtk_box_pack_start (GTK_BOX(window), results, TRUE, TRUE, GNOME_PAD_SMALL);
-
 	return window;
 }
 
@@ -1433,7 +1537,7 @@ main (int 	argc,
 	gtk_window_set_geometry_hints (GTK_WINDOW(interface.main_window), GTK_WIDGET(interface.main_window),
 				       &interface.geometry, GDK_HINT_MIN_SIZE);
 
-	gtk_window_set_type_hint (GTK_WINDOW(interface.main_window),GDK_WINDOW_TYPE_HINT_DIALOG);
+	gtk_window_set_position (GTK_WINDOW(interface.main_window), GTK_WIN_POS_CENTER);
 
 	gtk_window_set_focus (GTK_WINDOW(interface.main_window), 
 		GTK_WIDGET(gnome_entry_gtk_entry(GNOME_ENTRY(interface.file_is_named_entry))));
@@ -1448,6 +1552,8 @@ main (int 	argc,
 	
 	gtk_widget_show (interface.main_window);
 	
+	interface.icon_theme = gnome_icon_theme_new ();
+	
 	handle_popt_args ();
 		
 	gtk_main ();
